
// Load the data
queue()
    .defer(d3.csv, "data/artists.csv")
    .defer(d3.csv, "data/album_titles.csv")
    .await(compileData);

function compileData(error, artistData, albData) {
    // Make sure no error loading data
    if (error){
        return console.log(error);
    }

    // Create map between artist ID and the name
    let artistMap = {};
    artistData.forEach(function(artistObj) {
        artistMap[artistObj.artistID] = artistObj.artist;
    });

    // 347 albums
    // {albumID, title, artistID}
    console.log("album data", albData);
    console.log("artist data", artistData);

    // Iterate through the albums and count the occurences of an artistID
    // it's an array of {key, value}, 204 items
    let countByArtistID = d3.nest()
        .key(function(datum) { return datum.artistID})
        .rollup(function(leaves) {
            return leaves.length
        }) // the leaves are an array full of the objects with the corresponding key
        .entries(albData);

    console.log("artistId count", countByArtistID);

    // Sort the data in ascending order
    countByArtistID.sort(function(a,b) {
       return a.value - b.value;
    });

    // Splicing the array for display convenience. Not really necessary
    countByArtistID.splice(0, 180);

    // Now that the data has laoded, we can make the visualization
    createVis("chart-display-col", countByArtistID, artistMap, albData);


}

function createVis(parentElement, countData, artistMap, albData) {
    // Configure margins
    let margin = { top: 20, right: 20, bottom: 90, left: 30 };

    // Cofigure height and width of the visualization
    let width = $("#" + parentElement).width() - margin.left - margin.right,
        height = $("#" + parentElement).height() - margin.top - margin.bottom;

    // SVG drawing area
    let svg = d3.select("#" + parentElement).append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    // Create scales
    // Use ordinal scale since the x axis isn't numerical
    let xOrdinalScale = d3.scaleBand()
        .rangeRound([0, width])
        .padding(.5)
        .domain(d3.map(countData, function(datum){return artistMap[datum.key]}).keys());

    let yScale = d3.scaleLinear()
        .range([height - margin.bottom, margin.top]) // we could also set this to height and 0, but would need to translate axis
        .domain([0, d3.max(countData, function(datum){ return datum.value})]);

    // Now set up the axis
    let barYAxis = d3.axisLeft()
        .scale(yScale);

    let barXAxis = d3.axisBottom()
        .scale(xOrdinalScale);

    // Append the axes to the svg
    let barYAxisGroup = svg.append("g")
        .attr("class", "axis bar-y-axis")
        .call(barYAxis);

    let barXAxisGroup = svg.append("g")
        .attr("class", "axis bar-x-axis")
        .attr("transform", "translate(0," + (height - margin.bottom) + ")")
        .call(barXAxis)
        .selectAll("text")
        .attr("y", 0)
        .attr("x", 9)
        .attr("dy", ".35em")
        .attr("transform", "rotate(75)") // allows us to rotate the text
        .style("text-anchor", "start");;

    // Draw the bars
    let bars = svg.selectAll("rect")
        .data(countData)
        .enter()
        .append("rect")
        .attr("class","count-bar")
        .attr("fill", "red")
        .attr("width", xOrdinalScale.bandwidth())
        .attr("height", function(datum) {
            return yScale(0) - yScale(datum.value);
        })
        .attr("y", function(datum){
            return yScale(datum.value);
        })
        .attr("x", function(datum){
            return xOrdinalScale(artistMap[datum.key]);
        })
        .on("click", function(datum){
            getAlbumInfo(artistMap[datum.key], datum.key, albData);
        });

}

function getAlbumInfo(artistName, artistID, albumData){
    // Get artist name
    document.querySelector("#artist-span").innerHTML = artistName;

    // Construct list of albums for that artist
    let ul_list = "<ul>";
    albumData.forEach(function(album){
        if (album.artistID === artistID) {
            ul_list += "<li>" + album.title + "</li>";
        }
    })
    ul_list += "</ul>";

    // Put that html string into the dom
    document.querySelector("#albums").innerHTML = ul_list;

}



